@@ -37,7 +37,7 @@ using namespace nghttp2;
 namespace h2load {
 
 Http2Session::Http2Session(Client *client)
-    : client_(client), session_(nullptr) {}
+  : client_(client), session_(nullptr) {}
 
 Http2Session::~Http2Session() { nghttp2_session_del(session_); }
 
@@ -55,9 +55,11 @@ int on_header_callback(nghttp2_session *session, const nghttp2_frame *frame,
 
   if (client->worker->config->verbose) {
     std::cout << "[stream_id=" << frame->hd.stream_id << "] ";
-    std::cout.write(reinterpret_cast<const char *>(name), namelen);
+    std::cout.write(reinterpret_cast<const char *>(name),
+                    static_cast<std::streamsize>(namelen));
     std::cout << ": ";
-    std::cout.write(reinterpret_cast<const char *>(value), valuelen);
+    std::cout.write(reinterpret_cast<const char *>(value),
+                    static_cast<std::streamsize>(valuelen));
     std::cout << "\n";
   }
 
@@ -72,8 +74,8 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
   switch (frame->hd.type) {
   case NGHTTP2_HEADERS:
     client->worker->stats.bytes_head +=
-        frame->hd.length - frame->headers.padlen -
-        ((frame->hd.flags & NGHTTP2_FLAG_PRIORITY) ? 5 : 0);
+      frame->hd.length - frame->headers.padlen -
+      ((frame->hd.flags & NGHTTP2_FLAG_PRIORITY) ? 5 : 0);
     // fall through
   case NGHTTP2_DATA:
     if (frame->hd.flags & NGHTTP2_FLAG_END_STREAM) {
@@ -134,7 +136,7 @@ nghttp2_ssize file_read_callback(nghttp2_session *session, int32_t stream_id,
   assert(req_stat);
   ssize_t nread;
   while ((nread = pread(config->data_fd, buf, length, req_stat->data_offset)) ==
-             -1 &&
+           -1 &&
          errno == EINTR)
     ;
 
@@ -168,7 +170,9 @@ nghttp2_ssize send_callback(nghttp2_session *session, const uint8_t *data,
     return NGHTTP2_ERR_WOULDBLOCK;
   }
 
-  return wb.append(data, length);
+  wb.append(data, length);
+
+  return as_signed(length);
 }
 } // namespace
 
@@ -188,16 +192,16 @@ void Http2Session::on_connect() {
                                                        on_frame_recv_callback);
 
   nghttp2_session_callbacks_set_on_data_chunk_recv_callback(
-      callbacks, on_data_chunk_recv_callback);
+    callbacks, on_data_chunk_recv_callback);
 
   nghttp2_session_callbacks_set_on_stream_close_callback(
-      callbacks, on_stream_close_callback);
+    callbacks, on_stream_close_callback);
 
   nghttp2_session_callbacks_set_on_header_callback(callbacks,
                                                    on_header_callback);
 
   nghttp2_session_callbacks_set_before_frame_send_callback(
-      callbacks, before_frame_send_callback);
+    callbacks, before_frame_send_callback);
 
   nghttp2_session_callbacks_set_send_callback2(callbacks, send_callback);
 
@@ -210,7 +214,7 @@ void Http2Session::on_connect() {
 
   if (config->encoder_header_table_size != NGHTTP2_DEFAULT_HEADER_TABLE_SIZE) {
     nghttp2_option_set_max_deflate_dynamic_table_size(
-        opt, config->encoder_header_table_size);
+      opt, config->encoder_header_table_size);
   }
 
   nghttp2_session_client_new2(&session_, callbacks, client_, opt);
@@ -231,7 +235,7 @@ void Http2Session::on_connect() {
   }
   if (config->max_frame_size != 16_k) {
     iv[niv].settings_id = NGHTTP2_SETTINGS_MAX_FRAME_SIZE;
-    iv[niv].value = config->max_frame_size;
+    iv[niv].value = static_cast<uint32_t>(config->max_frame_size);
     ++niv;
   }
 
@@ -261,8 +265,8 @@ int Http2Session::submit_request() {
   nghttp2_data_provider2 prd{{0}, file_read_callback};
 
   auto stream_id =
-      nghttp2_submit_request2(session_, nullptr, nva.data(), nva.size(),
-                              config->data_fd == -1 ? nullptr : &prd, nullptr);
+    nghttp2_submit_request2(session_, nullptr, nva.data(), nva.size(),
+                            config->data_fd == -1 ? nullptr : &prd, nullptr);
   if (stream_id < 0) {
     return -1;
   }
