@@ -91,8 +91,9 @@ int Client::quic_recv_stream_data(uint32_t flags, int64_t stream_id,
     return -1;
   }
 
-  ngtcp2_conn_extend_max_stream_offset(quic.conn, stream_id, nconsumed);
-  ngtcp2_conn_extend_max_offset(quic.conn, nconsumed);
+  ngtcp2_conn_extend_max_stream_offset(quic.conn, stream_id,
+                                       static_cast<uint64_t>(nconsumed));
+  ngtcp2_conn_extend_max_offset(quic.conn, static_cast<uint64_t>(nconsumed));
 
   return 0;
 }
@@ -229,7 +230,8 @@ int Client::quic_extend_max_stream_data(int64_t stream_id) {
 namespace {
 int get_new_connection_id(ngtcp2_conn *conn, ngtcp2_cid *cid, uint8_t *token,
                           size_t cidlen, void *user_data) {
-  if (RAND_bytes(cid->data, cidlen) != 1) {
+  if (RAND_bytes(cid->data,
+                 static_cast<nghttp2_ssl_rand_length_type>(cidlen)) != 1) {
     return NGTCP2_ERR_CALLBACK_FAILURE;
   }
 
@@ -259,7 +261,8 @@ namespace {
 int generate_cid(ngtcp2_cid &dest) {
   dest.datalen = 8;
 
-  if (RAND_bytes(dest.data, dest.datalen) != 1) {
+  if (RAND_bytes(dest.data, static_cast<nghttp2_ssl_rand_length_type>(
+                              dest.datalen)) != 1) {
     return -1;
   }
 
@@ -269,9 +272,10 @@ int generate_cid(ngtcp2_cid &dest) {
 
 namespace {
 ngtcp2_tstamp quic_timestamp() {
-  return std::chrono::duration_cast<std::chrono::nanoseconds>(
-           std::chrono::steady_clock::now().time_since_epoch())
-    .count();
+  return static_cast<ngtcp2_tstamp>(
+    std::chrono::duration_cast<std::chrono::nanoseconds>(
+      std::chrono::steady_clock::now().time_since_epoch())
+      .count());
 }
 } // namespace
 
@@ -291,8 +295,12 @@ void Client::quic_write_qlog(const void *data, size_t datalen) {
 
 namespace {
 void rand(uint8_t *dest, size_t destlen, const ngtcp2_rand_ctx *rand_ctx) {
-  util::random_bytes(dest, dest + destlen,
-                     *static_cast<std::mt19937 *>(rand_ctx->native_handle));
+  auto rv =
+    RAND_bytes(dest, static_cast<nghttp2_ssl_rand_length_type>(destlen));
+  if (rv != 1) {
+    assert(0);
+    abort();
+  }
 }
 } // namespace
 
@@ -342,7 +350,22 @@ int Client::quic_init(const sockaddr *local_addr, socklen_t local_addrlen,
 
     SSL_set_app_data(ssl, &quic.conn_ref);
     SSL_set_connect_state(ssl);
+#if OPENSSL_3_5_0_API
+    if (ngtcp2_crypto_ossl_configure_client_session(ssl) != 0) {
+      std::cerr << "ngtcp2_crypto_ossl_configure_client_session failed"
+                << std::endl;
+      return -1;
+    }
+
+    rv = ngtcp2_crypto_ossl_ctx_new(&quic.ossl_ctx, ssl);
+    if (rv != 0) {
+      std::cerr << "ngtcp2_crypto_ossl_ctx_new failed with error code " << rv
+                << std::endl;
+      return -1;
+    }
+#else  // !OPENSSL_3_5_0_API
     SSL_set_quic_use_legacy_codepoint(ssl, 0);
+#endif // !OPENSSL_3_5_0_API
   }
 
   auto callbacks = ngtcp2_callbacks{
@@ -426,8 +449,8 @@ int Client::quic_init(const sockaddr *local_addr, socklen_t local_addrlen,
 
   ngtcp2_transport_params params;
   ngtcp2_transport_params_default(&params);
-  auto max_stream_data =
-    std::min((1 << 26) - 1, (1 << config->window_bits) - 1);
+  auto max_stream_data = static_cast<uint64_t>(
+    std::min((1 << 26) - 1, (1 << config->window_bits) - 1));
   params.initial_max_stream_data_bidi_local = max_stream_data;
   params.initial_max_stream_data_uni = max_stream_data;
   params.initial_max_data = (1 << config->connection_window_bits) - 1;
@@ -462,12 +485,20 @@ int Client::quic_init(const sockaddr *local_addr, socklen_t local_addrlen,
     return -1;
   }
 
+#if OPENSSL_3_5_0_API
+  ngtcp2_conn_set_tls_native_handle(quic.conn, quic.ossl_ctx);
+#else  // !OPENSSL_3_5_0_API
   ngtcp2_conn_set_tls_native_handle(quic.conn, ssl);
+#endif // !OPENSSL_3_5_0_API
 
   return 0;
 }
 
 void Client::quic_free() {
+#if OPENSSL_3_5_0_API
+  ngtcp2_crypto_ossl_ctx_del(quic.ossl_ctx);
+#endif // OPENSSL_3_5_0_API
+
   ngtcp2_conn_del(quic.conn);
   if (quic.qlog_file != nullptr) {
     fclose(quic.qlog_file);
@@ -493,7 +524,8 @@ void Client::quic_close_connection() {
   }
 
   write_udp(reinterpret_cast<sockaddr *>(ps.path.remote.addr),
-            ps.path.remote.addrlen, buf.data(), nwrite, 0);
+            ps.path.remote.addrlen, {buf.data(), static_cast<size_t>(nwrite)},
+            as_unsigned(nwrite));
 }
 
 int Client::quic_write_client_handshake(ngtcp2_encryption_level level,
@@ -583,7 +615,8 @@ int Client::read_quic() {
     assert(quic.conn);
 
     if (gso_size) {
-      worker->stats.udp_dgram_recv += (nread + gso_size - 1) / gso_size;
+      worker->stats.udp_dgram_recv +=
+        (as_unsigned(nread) + gso_size - 1) / gso_size;
     } else {
       ++worker->stats.udp_dgram_recv;
     }
@@ -591,7 +624,7 @@ int Client::read_quic() {
     auto path = ngtcp2_path{
       {
         &local_addr.su.sa,
-        static_cast<socklen_t>(local_addr.len),
+        local_addr.len,
       },
       {
         &su.sa,
@@ -662,11 +695,14 @@ int Client::write_quic() {
     ngtcp2_conn_get_max_tx_udp_payload_size(quic.conn);
   auto path_max_udp_payload_size =
     ngtcp2_conn_get_path_max_tx_udp_payload_size(quic.conn);
-  uint8_t *bufpos = quic.tx.data.get();
-  auto bufleft = std::max(ngtcp2_conn_get_send_quantum(quic.conn),
-                          path_max_udp_payload_size);
+  auto txbuf = std::span{quic.tx.data.get(),
+                         std::max(ngtcp2_conn_get_send_quantum(quic.conn),
+                                  path_max_udp_payload_size)};
+  auto buf = txbuf;
   ngtcp2_path_storage ps;
   size_t gso_size = 0;
+  auto pkt = std::span<const uint8_t>{};
+  auto extra_pkt = std::span<const uint8_t>{};
 
   ngtcp2_path_storage_zero(&ps);
 
@@ -694,11 +730,12 @@ int Client::write_quic() {
       flags |= NGTCP2_WRITE_STREAM_FLAG_FIN;
     }
 
-    auto buflen = bufleft >= max_udp_payload_size ? max_udp_payload_size
-                                                  : path_max_udp_payload_size;
+    auto buflen = buf.size() >= max_udp_payload_size
+                    ? max_udp_payload_size
+                    : path_max_udp_payload_size;
     auto nwrite = ngtcp2_conn_writev_stream(
-      quic.conn, &ps.path, nullptr, bufpos, buflen, &ndatalen, flags, stream_id,
-      reinterpret_cast<const ngtcp2_vec *>(v), vcnt, ts);
+      quic.conn, &ps.path, nullptr, buf.data(), buflen, &ndatalen, flags,
+      stream_id, reinterpret_cast<const ngtcp2_vec *>(v), vcnt, ts);
     if (nwrite < 0) {
       switch (nwrite) {
       case NGTCP2_ERR_STREAM_DATA_BLOCKED:
@@ -711,112 +748,81 @@ int Client::write_quic() {
         continue;
       case NGTCP2_ERR_WRITE_MORE:
         assert(ndatalen >= 0);
-        if (s->add_write_offset(stream_id, ndatalen) != 0) {
+        if (s->add_write_offset(stream_id, as_unsigned(ndatalen)) != 0) {
           return -1;
         }
         continue;
       }
 
-      ngtcp2_ccerr_set_liberr(&quic.last_error, nwrite, nullptr, 0);
+      ngtcp2_ccerr_set_liberr(&quic.last_error, static_cast<int>(nwrite),
+                              nullptr, 0);
       return -1;
-    } else if (ndatalen >= 0 && s->add_write_offset(stream_id, ndatalen) != 0) {
+    } else if (ndatalen >= 0 &&
+               s->add_write_offset(stream_id, as_unsigned(ndatalen)) != 0) {
       return -1;
     }
 
     quic_restart_pkt_timer();
 
     if (nwrite == 0) {
-      if (bufpos - quic.tx.data.get()) {
-        auto data = quic.tx.data.get();
-        auto datalen = bufpos - quic.tx.data.get();
-        rv = write_udp(ps.path.remote.addr, ps.path.remote.addrlen, data,
-                       datalen, gso_size);
-        if (rv == 1) {
-          on_send_blocked(ps.path.remote, data, datalen, gso_size);
-          signal_write();
-          return 0;
-        }
-      }
-      return 0;
-    }
-
-    auto first_pkt = bufpos == quic.tx.data.get();
-    (void)first_pkt;
-
-    bufpos += nwrite;
-    bufleft -= nwrite;
-
-#ifdef UDP_SEGMENT
-    if (worker->config->no_udp_gso) {
-#endif // UDP_SEGMENT
-      auto data = quic.tx.data.get();
-      auto datalen = bufpos - quic.tx.data.get();
-      rv = write_udp(ps.path.remote.addr, ps.path.remote.addrlen, data, datalen,
-                     0);
-      if (rv == 1) {
-        on_send_blocked(ps.path.remote, data, datalen, 0);
-        signal_write();
+      pkt = std::span{std::ranges::begin(txbuf), std::ranges::begin(buf)};
+      if (pkt.empty()) {
         return 0;
       }
 
-      if (bufleft < path_max_udp_payload_size) {
-        signal_write();
-        return 0;
-      }
-
-      bufpos = quic.tx.data.get();
-
-#ifdef UDP_SEGMENT
-      continue;
+      break;
     }
-#endif // UDP_SEGMENT
 
-#ifdef UDP_SEGMENT
-    if (first_pkt) {
-      gso_size = nwrite;
+    auto last_pkt_pos = std::ranges::begin(buf);
+
+    buf = buf.subspan(as_unsigned(nwrite));
+
+    if (last_pkt_pos == std::ranges::begin(txbuf)) {
+      gso_size = as_unsigned(nwrite);
     } else if (static_cast<size_t>(nwrite) > gso_size ||
                (gso_size > path_max_udp_payload_size &&
                 static_cast<size_t>(nwrite) != gso_size)) {
-      auto data = quic.tx.data.get();
-      auto datalen = bufpos - quic.tx.data.get() - nwrite;
-      rv = write_udp(ps.path.remote.addr, ps.path.remote.addrlen, data, datalen,
-                     gso_size);
-      if (rv == 1) {
-        on_send_blocked(ps.path.remote, data, datalen, gso_size);
-        on_send_blocked(ps.path.remote, bufpos - nwrite, nwrite, 0);
-      } else {
-        auto data = bufpos - nwrite;
-        rv = write_udp(ps.path.remote.addr, ps.path.remote.addrlen, data,
-                       nwrite, 0);
-        if (rv == 1) {
-          on_send_blocked(ps.path.remote, data, nwrite, 0);
-        }
-      }
-
-      signal_write();
-      return 0;
+      pkt = std::span{std::ranges::begin(txbuf), last_pkt_pos};
+      extra_pkt = std::span{last_pkt_pos, std::ranges::begin(buf)};
+      break;
     }
 
     // Assume that the path does not change.
-    if (bufleft < path_max_udp_payload_size ||
+    if (buf.size() < path_max_udp_payload_size ||
         static_cast<size_t>(nwrite) < gso_size) {
-      auto data = quic.tx.data.get();
-      auto datalen = bufpos - quic.tx.data.get();
-      rv = write_udp(ps.path.remote.addr, ps.path.remote.addrlen, data, datalen,
-                     gso_size);
-      if (rv == 1) {
-        on_send_blocked(ps.path.remote, data, datalen, gso_size);
-      }
-      signal_write();
-      return 0;
+      pkt = std::span{std::ranges::begin(txbuf), std::ranges::begin(buf)};
+      break;
     }
-#endif // UDP_SEGMENT
   }
+
+  if (write_udp_or_blocked(ps.path, pkt, gso_size) != 0) {
+    if (!extra_pkt.empty()) {
+      on_send_blocked(ps.path.remote, extra_pkt, extra_pkt.size());
+    }
+  } else if (!extra_pkt.empty()) {
+    write_udp_or_blocked(ps.path, extra_pkt, extra_pkt.size());
+  }
+
+  ngtcp2_conn_update_pkt_tx_time(quic.conn, ts);
+
+  return 0;
+}
+
+int Client::write_udp_or_blocked(const ngtcp2_path &path,
+                                 std::span<const uint8_t> data,
+                                 size_t gso_size) {
+  auto rest = write_udp(path.remote.addr, path.remote.addrlen, data, gso_size);
+  if (!rest.empty()) {
+    on_send_blocked(path.remote, data, gso_size);
+
+    return -1;
+  }
+
+  return 0;
 }
 
 void Client::on_send_blocked(const ngtcp2_addr &remote_addr,
-                             const uint8_t *data, size_t datalen,
-                             size_t gso_size) {
+                             std::span<const uint8_t> data, size_t gso_size) {
   assert(quic.tx.num_blocked || !quic.tx.send_blocked);
   assert(quic.tx.num_blocked < 2);
 
@@ -828,22 +834,23 @@ void Client::on_send_blocked(const ngtcp2_addr &remote_addr,
 
   p.remote_addr.len = remote_addr.addrlen;
   p.data = data;
-  p.datalen = datalen;
   p.gso_size = gso_size;
+
+  signal_write();
 }
 
 int Client::send_blocked_packet() {
-  int rv;
-
   assert(quic.tx.send_blocked);
 
   for (; quic.tx.num_blocked_sent < quic.tx.num_blocked;
        ++quic.tx.num_blocked_sent) {
     auto &p = quic.tx.blocked[quic.tx.num_blocked_sent];
 
-    rv = write_udp(&p.remote_addr.su.sa, p.remote_addr.len, p.data, p.datalen,
-                   p.gso_size);
-    if (rv == 1) {
+    auto rest =
+      write_udp(&p.remote_addr.su.sa, p.remote_addr.len, p.data, p.gso_size);
+    if (!rest.empty()) {
+      p.data = rest;
+
       signal_write();
 
       return 0;
