@@ -39,38 +39,32 @@
 namespace shrpx {
 
 namespace {
-// List of API endpoints
-const std::array<APIEndpoint, 2> &apis() {
-  static const auto apis = new std::array<APIEndpoint, 2>{
-      APIEndpoint{
-          StringRef::from_lit("/api/v1beta1/backendconfig"),
-          true,
-          (1 << API_METHOD_POST) | (1 << API_METHOD_PUT),
-          &APIDownstreamConnection::handle_backendconfig,
-      },
-      APIEndpoint{
-          StringRef::from_lit("/api/v1beta1/configrevision"),
-          true,
-          (1 << API_METHOD_GET),
-          &APIDownstreamConnection::handle_configrevision,
-      },
-  };
+const auto backendconfig_endpoint = APIEndpoint{
+  "/api/v1beta1/backendconfig"sv,
+  true,
+  (1 << API_METHOD_POST) | (1 << API_METHOD_PUT),
+  &APIDownstreamConnection::handle_backendconfig,
+};
 
-  return *apis;
-}
+const auto configrevision_endpoint = APIEndpoint{
+  "/api/v1beta1/configrevision"sv,
+  true,
+  (1 << API_METHOD_GET),
+  &APIDownstreamConnection::handle_configrevision,
+};
 } // namespace
 
 namespace {
 // The method string.  This must be same order of APIMethod.
-constexpr StringRef API_METHOD_STRING[] = {
-    StringRef::from_lit("GET"),
-    StringRef::from_lit("POST"),
-    StringRef::from_lit("PUT"),
+constexpr std::string_view API_METHOD_STRING[] = {
+  "GET"sv,
+  "POST"sv,
+  "PUT"sv,
 };
 } // namespace
 
 APIDownstreamConnection::APIDownstreamConnection(Worker *worker)
-    : worker_(worker), api_(nullptr), fd_(-1), shutdown_read_(false) {}
+  : worker_(worker), api_(nullptr), fd_(-1), shutdown_read_(false) {}
 
 APIDownstreamConnection::~APIDownstreamConnection() {
   if (fd_ != -1) {
@@ -97,7 +91,7 @@ void APIDownstreamConnection::detach_downstream(Downstream *downstream) {
 
 int APIDownstreamConnection::send_reply(unsigned int http_status,
                                         APIStatusCode api_status,
-                                        const StringRef &data) {
+                                        const std::string_view &data) {
   shutdown_read_ = true;
 
   auto upstream = downstream_->get_upstream();
@@ -108,56 +102,55 @@ int APIDownstreamConnection::send_reply(unsigned int http_status,
 
   auto &balloc = downstream_->get_block_allocator();
 
-  StringRef api_status_str;
+  std::string_view api_status_str;
 
   switch (api_status) {
   case APIStatusCode::SUCCESS:
-    api_status_str = StringRef::from_lit("Success");
+    api_status_str = "Success"sv;
     break;
   case APIStatusCode::FAILURE:
-    api_status_str = StringRef::from_lit("Failure");
+    api_status_str = "Failure"sv;
     break;
   default:
     assert(0);
   }
 
-  constexpr auto M1 = StringRef::from_lit("{\"status\":\"");
-  constexpr auto M2 = StringRef::from_lit("\",\"code\":");
-  constexpr auto M3 = StringRef::from_lit("}");
+  constexpr auto M1 = "{\"status\":\""sv;
+  constexpr auto M2 = "\",\"code\":"sv;
+  constexpr auto M3 = "}"sv;
 
   // 3 is the number of digits in http_status, assuming it is 3 digits
   // number.
-  auto buflen = M1.size() + M2.size() + M3.size() + data.size() +
-                api_status_str.size() + 3;
+  auto buflen =
+    M1.size() + M2.size() + M3.size() + data.size() + api_status_str.size() + 3;
 
   auto buf = make_byte_ref(balloc, buflen);
-  auto p = buf.base;
+  auto p = std::ranges::begin(buf);
 
-  p = std::copy(std::begin(M1), std::end(M1), p);
-  p = std::copy(std::begin(api_status_str), std::end(api_status_str), p);
-  p = std::copy(std::begin(M2), std::end(M2), p);
-  p = util::utos(p, http_status);
-  p = std::copy(std::begin(data), std::end(data), p);
-  p = std::copy(std::begin(M3), std::end(M3), p);
+  p = std::ranges::copy(M1, p).out;
+  p = std::ranges::copy(api_status_str, p).out;
+  p = std::ranges::copy(M2, p).out;
+  p = util::utos(http_status, p);
+  p = std::ranges::copy(data, p).out;
+  p = std::ranges::copy(M3, p).out;
 
-  buf.len = p - buf.base;
+  buf = buf.first(as_unsigned(p - std::ranges::begin(buf)));
 
-  auto content_length = util::make_string_ref_uint(balloc, buf.len);
+  auto content_length = util::make_string_ref_uint(balloc, buf.size());
 
-  resp.fs.add_header_token(StringRef::from_lit("content-length"),
-                           content_length, false, http2::HD_CONTENT_LENGTH);
+  resp.fs.add_header_token("content-length"sv, content_length, false,
+                           http2::HD_CONTENT_LENGTH);
 
   switch (http_status) {
   case 400:
   case 405:
   case 413:
-    resp.fs.add_header_token(StringRef::from_lit("connection"),
-                             StringRef::from_lit("close"), false,
+    resp.fs.add_header_token("connection"sv, "close"sv, false,
                              http2::HD_CONNECTION);
     break;
   }
 
-  if (upstream->send_reply(downstream_, buf.base, buf.len) != 0) {
+  if (upstream->send_reply(downstream_, buf.data(), buf.size()) != 0) {
     return -1;
   }
 
@@ -165,13 +158,13 @@ int APIDownstreamConnection::send_reply(unsigned int http_status,
 }
 
 namespace {
-const APIEndpoint *lookup_api(const StringRef &path) {
+const APIEndpoint *lookup_api(const std::string_view &path) {
   switch (path.size()) {
   case 26:
     switch (path[25]) {
     case 'g':
-      if (util::streq_l("/api/v1beta1/backendconfi", std::begin(path), 25)) {
-        return &apis()[0];
+      if (util::streq("/api/v1beta1/backendconfi"sv, path.substr(0, 25))) {
+        return &backendconfig_endpoint;
       }
       break;
     }
@@ -179,8 +172,8 @@ const APIEndpoint *lookup_api(const StringRef &path) {
   case 27:
     switch (path[26]) {
     case 'n':
-      if (util::streq_l("/api/v1beta1/configrevisio", std::begin(path), 26)) {
-        return &apis()[1];
+      if (util::streq("/api/v1beta1/configrevisio"sv, path.substr(0, 26))) {
+        return &configrevision_endpoint;
       }
       break;
     }
@@ -193,9 +186,8 @@ const APIEndpoint *lookup_api(const StringRef &path) {
 int APIDownstreamConnection::push_request_headers() {
   auto &req = downstream_->request();
 
-  auto path =
-      StringRef{std::begin(req.path),
-                std::find(std::begin(req.path), std::end(req.path), '?')};
+  auto path = std::string_view{std::ranges::begin(req.path),
+                               std::ranges::find(req.path, '?')};
 
   api_ = lookup_api(path);
 
@@ -283,20 +275,20 @@ int APIDownstreamConnection::error_method_not_allowed() {
   auto &balloc = downstream_->get_block_allocator();
 
   auto iov = make_byte_ref(balloc, len + 1);
-  auto p = iov.base;
+  auto p = std::ranges::begin(iov);
   for (uint8_t i = 0; i < API_METHOD_MAX; ++i) {
     if (api_->allowed_methods & (1 << i)) {
       auto &s = API_METHOD_STRING[i];
-      p = std::copy(std::begin(s), std::end(s), p);
-      p = std::copy_n(", ", 2, p);
+      p = std::ranges::copy(s, p).out;
+      p = std::ranges::copy(", "sv, p).out;
     }
   }
 
   p -= 2;
   *p = '\0';
 
-  resp.fs.add_header_token(StringRef::from_lit("allow"), StringRef{iov.base, p},
-                           false, -1);
+  resp.fs.add_header_token(
+    "allow"sv, as_string_view(std::ranges::begin(iov), p), false, -1);
   return send_reply(405, APIStatusCode::FAILURE);
 }
 
@@ -350,9 +342,11 @@ int APIDownstreamConnection::handle_backendconfig() {
     return 0;
   }
 
-  auto rp = mmap(nullptr, req.recv_body_length, PROT_READ, MAP_SHARED, fd_, 0);
+  auto rp = mmap(nullptr, static_cast<size_t>(req.recv_body_length), PROT_READ,
+                 MAP_SHARED, fd_, 0);
   if (rp == reinterpret_cast<void *>(-1)) {
     send_reply(500, APIStatusCode::FAILURE);
+    return 0;
   }
 
   auto unmapper = defer(munmap, rp, req.recv_body_length);
@@ -371,13 +365,13 @@ int APIDownstreamConnection::handle_backendconfig() {
   downstreamconf->response_buffer_size = src->response_buffer_size;
   downstreamconf->family = src->family;
 
-  std::set<StringRef> include_set;
-  std::map<StringRef, size_t> pattern_addr_indexer;
+  std::unordered_set<std::string_view> include_set;
+  std::unordered_map<std::string_view, size_t> pattern_addr_indexer;
 
-  for (auto first = reinterpret_cast<const uint8_t *>(rp),
+  for (auto first = reinterpret_cast<const char *>(rp),
             last = first + req.recv_body_length;
        first != last;) {
-    auto eol = std::find(first, last, '\n');
+    auto eol = std::ranges::find(first, last, '\n');
     if (eol == last) {
       break;
     }
@@ -387,16 +381,16 @@ int APIDownstreamConnection::handle_backendconfig() {
       continue;
     }
 
-    auto eq = std::find(first, eol, '=');
+    auto eq = std::ranges::find(first, eol, '=');
     if (eq == eol) {
       send_reply(400, APIStatusCode::FAILURE);
       return 0;
     }
 
-    auto opt = StringRef{first, eq};
-    auto optval = StringRef{eq + 1, eol};
+    auto opt = std::string_view{first, eq};
+    auto optval = std::string_view{eq + 1, eol};
 
-    auto optid = option_lookup_token(opt.c_str(), opt.size());
+    auto optid = option_lookup_token(opt);
 
     switch (optid) {
     case SHRPX_OPTID_BACKEND:
@@ -441,9 +435,8 @@ int APIDownstreamConnection::handle_configrevision() {
   //     "configRevision": N
   //   }
   auto data = concat_string_ref(
-      balloc, StringRef::from_lit(R"(,"data":{"configRevision":)"),
-      util::make_string_ref_uint(balloc, config->config_revision),
-      StringRef::from_lit("}"));
+    balloc, R"(,"data":{"configRevision":)"sv,
+    util::make_string_ref_uint(balloc, config->config_revision), "}"sv);
 
   send_reply(200, APIStatusCode::SUCCESS, data);
 
@@ -462,7 +455,7 @@ int APIDownstreamConnection::on_read() { return 0; }
 
 int APIDownstreamConnection::on_write() { return 0; }
 
-void APIDownstreamConnection::on_upstream_change(Upstream *uptream) {}
+void APIDownstreamConnection::on_upstream_change(Upstream *upstream) {}
 
 bool APIDownstreamConnection::poolable() const { return false; }
 
