@@ -31,7 +31,14 @@
 
 #include <vector>
 
-#include <openssl/err.h>
+#include "ssl_compat.h"
+
+#ifdef NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/err.h>
+#else // !NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <openssl/err.h>
+#endif // !NGHTTP2_OPENSSL_IS_WOLFSSL
 
 #include "shrpx_upstream.h"
 #include "shrpx_downstream.h"
@@ -182,25 +189,25 @@ Http2Session::Http2Session(struct ev_loop *loop, SSL_CTX *ssl_ctx,
                            Worker *worker,
                            const std::shared_ptr<DownstreamAddrGroup> &group,
                            DownstreamAddr *addr)
-    : dlnext(nullptr),
-      dlprev(nullptr),
-      conn_(loop, -1, nullptr, worker->get_mcpool(),
-            group->shared_addr->timeout.write, group->shared_addr->timeout.read,
-            {}, {}, writecb, readcb, timeoutcb, this,
-            get_config()->tls.dyn_rec.warmup_threshold,
-            get_config()->tls.dyn_rec.idle_timeout, Proto::HTTP2),
-      wb_(worker->get_mcpool()),
-      worker_(worker),
-      ssl_ctx_(ssl_ctx),
-      group_(group),
-      addr_(addr),
-      session_(nullptr),
-      raddr_(nullptr),
-      state_(Http2SessionState::DISCONNECTED),
-      connection_check_state_(ConnectionCheck::NONE),
-      freelist_zone_(FreelistZone::NONE),
-      settings_recved_(false),
-      allow_connect_proto_(false) {
+  : dlnext(nullptr),
+    dlprev(nullptr),
+    conn_(loop, -1, nullptr, worker->get_mcpool(),
+          group->shared_addr->timeout.write, group->shared_addr->timeout.read,
+          {}, {}, writecb, readcb, timeoutcb, this,
+          get_config()->tls.dyn_rec.warmup_threshold,
+          get_config()->tls.dyn_rec.idle_timeout, Proto::HTTP2),
+    wb_(worker->get_mcpool()),
+    worker_(worker),
+    ssl_ctx_(ssl_ctx),
+    group_(group),
+    addr_(addr),
+    session_(nullptr),
+    raddr_(nullptr),
+    state_(Http2SessionState::DISCONNECTED),
+    connection_check_state_(ConnectionCheck::NONE),
+    freelist_zone_(FreelistZone::NONE),
+    settings_recved_(false),
+    allow_connect_proto_(false) {
   read_ = write_ = &Http2Session::noop;
 
   on_read_ = &Http2Session::read_noop;
@@ -305,19 +312,19 @@ int Http2Session::disconnect(bool hard) {
 
 int Http2Session::resolve_name() {
   auto dns_query = std::make_unique<DNSQuery>(
-      addr_->host, [this](DNSResolverStatus status, const Address *result) {
-        int rv;
+    addr_->host, [this](DNSResolverStatus status, const Address *result) {
+      int rv;
 
-        if (status == DNSResolverStatus::OK) {
-          *resolved_addr_ = *result;
-          util::set_port(*this->resolved_addr_, this->addr_->port);
-        }
+      if (status == DNSResolverStatus::OK) {
+        *resolved_addr_ = *result;
+        util::set_port(*this->resolved_addr_, this->addr_->port);
+      }
 
-        rv = this->initiate_connection();
-        if (rv != 0) {
-          delete this;
-        }
-      });
+      rv = this->initiate_connection();
+      if (rv != 0) {
+        delete this;
+      }
+    });
   resolved_addr_ = std::make_unique<Address>();
   auto dns_tracker = worker_->get_dns_tracker();
   switch (dns_tracker->resolve(resolved_addr_.get(), dns_query.get())) {
@@ -342,29 +349,7 @@ int htp_hdrs_completecb(llhttp_t *htp);
 
 namespace {
 constexpr llhttp_settings_t htp_hooks = {
-    nullptr,             // llhttp_cb      on_message_begin;
-    nullptr,             // llhttp_data_cb on_url;
-    nullptr,             // llhttp_data_cb on_status;
-    nullptr,             // llhttp_data_cb on_method;
-    nullptr,             // llhttp_data_cb on_version;
-    nullptr,             // llhttp_data_cb on_header_field;
-    nullptr,             // llhttp_data_cb on_header_value;
-    nullptr,             // llhttp_data_cb on_chunk_extension_name;
-    nullptr,             // llhttp_data_cb on_chunk_extension_value;
-    htp_hdrs_completecb, // llhttp_cb      on_headers_complete;
-    nullptr,             // llhttp_data_cb on_body;
-    nullptr,             // llhttp_cb      on_message_complete;
-    nullptr,             // llhttp_cb      on_url_complete;
-    nullptr,             // llhttp_cb      on_status_complete;
-    nullptr,             // llhttp_cb      on_method_complete;
-    nullptr,             // llhttp_cb      on_version_complete;
-    nullptr,             // llhttp_cb      on_header_field_complete;
-    nullptr,             // llhttp_cb      on_header_value_complete;
-    nullptr,             // llhttp_cb      on_chunk_extension_name_complete;
-    nullptr,             // llhttp_cb      on_chunk_extension_value_complete;
-    nullptr,             // llhttp_cb      on_chunk_header;
-    nullptr,             // llhttp_cb      on_chunk_complete;
-    nullptr,             // llhttp_cb      on_reset;
+  .on_headers_complete = htp_hdrs_completecb,
 };
 } // namespace
 
@@ -378,7 +363,7 @@ int Http2Session::initiate_connection() {
     if (worker_blocker->blocked()) {
       if (LOG_ENABLED(INFO)) {
         SSLOG(INFO, this)
-            << "Worker wide backend connection was blocked temporarily";
+          << "Worker wide backend connection was blocked temporarily";
       }
       return -1;
     }
@@ -465,14 +450,13 @@ int Http2Session::initiate_connection() {
         conn_.set_ssl(ssl);
         conn_.tls.client_session_cache = &addr_->tls_session_cache;
 
-        auto sni_name =
-            addr_->sni.empty() ? StringRef{addr_->host} : StringRef{addr_->sni};
+        auto sni_name = addr_->sni.empty() ? addr_->host : addr_->sni;
 
-        if (!util::numeric_host(sni_name.c_str())) {
+        if (!util::numeric_host(sni_name.data())) {
           // TLS extensions: SNI. There is no documentation about the return
           // code for this function (actually this is macro wrapping SSL_ctrl
           // at the time of this writing).
-          SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name.c_str());
+          SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name.data());
         }
 
         auto tls_session = tls::reuse_tls_session(addr_->tls_session_cache);
@@ -519,8 +503,8 @@ int Http2Session::initiate_connection() {
         if (conn_.fd == -1) {
           auto error = errno;
           SSLOG(WARN, this)
-              << "socket() failed; addr=" << util::to_numeric_addr(raddr_)
-              << ", errno=" << error;
+            << "socket() failed; addr=" << util::to_numeric_addr(raddr_)
+            << ", errno=" << error;
 
           worker_blocker->on_failure();
           return -1;
@@ -530,12 +514,12 @@ int Http2Session::initiate_connection() {
 
         rv = connect(conn_.fd,
                      // TODO maybe not thread-safe?
-                     const_cast<sockaddr *>(&raddr_->su.sa), raddr_->len);
+                     &raddr_->su.sa, raddr_->len);
         if (rv != 0 && errno != EINPROGRESS) {
           auto error = errno;
           SSLOG(WARN, this)
-              << "connect() failed; addr=" << util::to_numeric_addr(raddr_)
-              << ", errno=" << error;
+            << "connect() failed; addr=" << util::to_numeric_addr(raddr_)
+            << ", errno=" << error;
 
           downstream_failure(addr_, raddr_);
           return -1;
@@ -584,8 +568,8 @@ int Http2Session::initiate_connection() {
         if (conn_.fd == -1) {
           auto error = errno;
           SSLOG(WARN, this)
-              << "socket() failed; addr=" << util::to_numeric_addr(raddr_)
-              << ", errno=" << error;
+            << "socket() failed; addr=" << util::to_numeric_addr(raddr_)
+            << ", errno=" << error;
 
           worker_blocker->on_failure();
           return -1;
@@ -593,13 +577,12 @@ int Http2Session::initiate_connection() {
 
         worker_blocker->on_success();
 
-        rv = connect(conn_.fd, const_cast<sockaddr *>(&raddr_->su.sa),
-                     raddr_->len);
+        rv = connect(conn_.fd, &raddr_->su.sa, raddr_->len);
         if (rv != 0 && errno != EINPROGRESS) {
           auto error = errno;
           SSLOG(WARN, this)
-              << "connect() failed; addr=" << util::to_numeric_addr(raddr_)
-              << ", errno=" << error;
+            << "connect() failed; addr=" << util::to_numeric_addr(raddr_)
+            << ", errno=" << error;
 
           downstream_failure(addr_, raddr_);
           return -1;
@@ -693,7 +676,7 @@ int Http2Session::downstream_connect_proxy() {
   }
 
   std::string req = "CONNECT ";
-  req.append(addr_->hostport.c_str(), addr_->hostport.size());
+  req.append(addr_->hostport.data(), addr_->hostport.size());
   if (addr_->port == 80 || addr_->port == 443) {
     req += ':';
     req += util::utos(addr_->port);
@@ -704,7 +687,7 @@ int Http2Session::downstream_connect_proxy() {
   const auto &proxy = get_config()->downstream_http_proxy;
   if (!proxy.userinfo.empty()) {
     req += "Proxy-Authorization: Basic ";
-    req += base64::encode(std::begin(proxy.userinfo), std::end(proxy.userinfo));
+    req += base64::encode(proxy.userinfo);
     req += "\r\n";
   }
   req += "\r\n";
@@ -725,7 +708,7 @@ void Http2Session::add_downstream_connection(Http2DownstreamConnection *dconn) {
 }
 
 void Http2Session::remove_downstream_connection(
-    Http2DownstreamConnection *dconn) {
+  Http2DownstreamConnection *dconn) {
   --addr_->num_dconn;
   dconns_.remove(dconn);
   dconn->detach_stream_data();
@@ -760,8 +743,8 @@ int Http2Session::submit_request(Http2DownstreamConnection *dconn,
   auto sd = std::make_unique<StreamData>();
   sd->dlnext = sd->dlprev = nullptr;
   // TODO Specify nullptr to pri_spec for now
-  auto stream_id = nghttp2_submit_request2(session_, nullptr, nva, nvlen,
-                                           data_prd, sd.get());
+  auto stream_id =
+    nghttp2_submit_request2(session_, nullptr, nva, nvlen, data_prd, sd.get());
   if (stream_id < 0) {
     SSLOG(FATAL, this) << "nghttp2_submit_request2() failed: "
                        << nghttp2_strerror(stream_id);
@@ -796,8 +779,8 @@ nghttp2_session *Http2Session::get_session() const { return session_; }
 int Http2Session::resume_data(Http2DownstreamConnection *dconn) {
   assert(state_ == Http2SessionState::CONNECTED);
   auto downstream = dconn->get_downstream();
-  int rv = nghttp2_session_resume_data(session_,
-                                       downstream->get_downstream_stream_id());
+  int rv = nghttp2_session_resume_data(
+    session_, static_cast<int32_t>(downstream->get_downstream_stream_id()));
   switch (rv) {
   case 0:
   case NGHTTP2_ERR_INVALID_ARGUMENT:
@@ -828,11 +811,11 @@ int on_stream_close_callback(nghttp2_session *session, int32_t stream_id,
   auto http2session = static_cast<Http2Session *>(user_data);
   if (LOG_ENABLED(INFO)) {
     SSLOG(INFO, http2session)
-        << "Stream stream_id=" << stream_id
-        << " is being closed with error code " << error_code;
+      << "Stream stream_id=" << stream_id << " is being closed with error code "
+      << error_code;
   }
   auto sd = static_cast<StreamData *>(
-      nghttp2_session_get_stream_user_data(session, stream_id));
+    nghttp2_session_get_stream_user_data(session, stream_id));
   if (sd == 0) {
     // We might get this close callback when pushed streams are
     // closed.
@@ -853,7 +836,7 @@ int on_stream_close_callback(nghttp2_session *session, int32_t stream_id,
       upstream->cancel_premature_downstream(downstream);
     } else {
       if (downstream->get_upgraded() && downstream->get_response_state() ==
-                                            DownstreamState::HEADER_COMPLETE) {
+                                          DownstreamState::HEADER_COMPLETE) {
         // For tunneled connection, we have to submit RST_STREAM to
         // upstream *after* whole response body is sent. We just set
         // MSG_COMPLETE here. Upstream will take care of that.
@@ -873,7 +856,7 @@ int on_stream_close_callback(nghttp2_session *session, int32_t stream_id,
       }
       if (downstream->get_response_state() == DownstreamState::MSG_RESET &&
           downstream->get_response_rst_stream_error_code() ==
-              NGHTTP2_NO_ERROR) {
+            NGHTTP2_NO_ERROR) {
         downstream->set_response_rst_stream_error_code(error_code);
       }
       call_downstream_readcb(http2session, downstream);
@@ -903,7 +886,7 @@ int on_header_callback2(nghttp2_session *session, const nghttp2_frame *frame,
                         uint8_t flags, void *user_data) {
   auto http2session = static_cast<Http2Session *>(user_data);
   auto sd = static_cast<StreamData *>(
-      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+    nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
   if (!sd || !sd->dconn) {
     return 0;
   }
@@ -921,13 +904,13 @@ int on_header_callback2(nghttp2_session *session, const nghttp2_frame *frame,
                    !downstream->get_expect_final_response();
 
     if (resp.fs.buffer_size() + namebuf.len + valuebuf.len >
-            httpconf.response_header_field_buffer ||
+          httpconf.response_header_field_buffer ||
         resp.fs.num_fields() >= httpconf.max_response_header_fields) {
       if (LOG_ENABLED(INFO)) {
         DLOG(INFO, downstream)
-            << "Too large or many header field size="
-            << resp.fs.buffer_size() + namebuf.len + valuebuf.len
-            << ", num=" << resp.fs.num_fields() + 1;
+          << "Too large or many header field size="
+          << resp.fs.buffer_size() + namebuf.len + valuebuf.len
+          << ", num=" << resp.fs.num_fields() + 1;
       }
 
       if (trailer) {
@@ -939,7 +922,9 @@ int on_header_callback2(nghttp2_session *session, const nghttp2_frame *frame,
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
     }
 
-    auto token = http2::lookup_token(namebuf.base, namebuf.len);
+    auto nameref = as_string_view(namebuf.base, namebuf.len);
+    auto valueref = as_string_view(valuebuf.base, valuebuf.len);
+    auto token = http2::lookup_token(nameref);
     auto no_index = flags & NGHTTP2_NV_FLAG_NO_INDEX;
 
     downstream->add_rcbuf(name);
@@ -947,21 +932,17 @@ int on_header_callback2(nghttp2_session *session, const nghttp2_frame *frame,
 
     if (trailer) {
       // just store header fields for trailer part
-      resp.fs.add_trailer_token(StringRef{namebuf.base, namebuf.len},
-                                StringRef{valuebuf.base, valuebuf.len},
-                                no_index, token);
+      resp.fs.add_trailer_token(nameref, valueref, no_index, token);
       return 0;
     }
 
-    resp.fs.add_header_token(StringRef{namebuf.base, namebuf.len},
-                             StringRef{valuebuf.base, valuebuf.len}, no_index,
-                             token);
+    resp.fs.add_header_token(nameref, valueref, no_index, token);
     return 0;
   }
   case NGHTTP2_PUSH_PROMISE: {
     auto promised_stream_id = frame->push_promise.promised_stream_id;
     auto promised_sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, promised_stream_id));
+      nghttp2_session_get_stream_user_data(session, promised_stream_id));
     if (!promised_sd || !promised_sd->dconn) {
       http2session->submit_rst_stream(promised_stream_id, NGHTTP2_CANCEL);
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
@@ -978,13 +959,13 @@ int on_header_callback2(nghttp2_session *session, const nghttp2_frame *frame,
 
     // We use request header limit for PUSH_PROMISE
     if (promised_req.fs.buffer_size() + namebuf.len + valuebuf.len >
-            httpconf.request_header_field_buffer ||
+          httpconf.request_header_field_buffer ||
         promised_req.fs.num_fields() >= httpconf.max_request_header_fields) {
       if (LOG_ENABLED(INFO)) {
         DLOG(INFO, downstream)
-            << "Too large or many header field size="
-            << promised_req.fs.buffer_size() + namebuf.len + valuebuf.len
-            << ", num=" << promised_req.fs.num_fields() + 1;
+          << "Too large or many header field size="
+          << promised_req.fs.buffer_size() + namebuf.len + valuebuf.len
+          << ", num=" << promised_req.fs.num_fields() + 1;
       }
 
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
@@ -993,9 +974,10 @@ int on_header_callback2(nghttp2_session *session, const nghttp2_frame *frame,
     promised_downstream->add_rcbuf(name);
     promised_downstream->add_rcbuf(value);
 
-    auto token = http2::lookup_token(namebuf.base, namebuf.len);
-    promised_req.fs.add_header_token(StringRef{namebuf.base, namebuf.len},
-                                     StringRef{valuebuf.base, valuebuf.len},
+    auto nameref = as_string_view(namebuf.base, namebuf.len);
+    auto valueref = as_string_view(valuebuf.base, valuebuf.len);
+    auto token = http2::lookup_token(nameref);
+    promised_req.fs.add_header_token(nameref, valueref,
                                      flags & NGHTTP2_NV_FLAG_NO_INDEX, token);
 
     return 0;
@@ -1013,7 +995,7 @@ int on_invalid_header_callback2(nghttp2_session *session,
                                 void *user_data) {
   auto http2session = static_cast<Http2Session *>(user_data);
   auto sd = static_cast<StreamData *>(
-      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+    nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
   if (!sd || !sd->dconn) {
     return 0;
   }
@@ -1031,10 +1013,10 @@ int on_invalid_header_callback2(nghttp2_session *session,
     auto valuebuf = nghttp2_rcbuf_get_buf(value);
 
     SSLOG(INFO, http2session)
-        << "Invalid header field for stream_id=" << stream_id
-        << " in frame type=" << static_cast<uint32_t>(frame->hd.type)
-        << ": name=[" << StringRef{namebuf.base, namebuf.len} << "], value=["
-        << StringRef{valuebuf.base, valuebuf.len} << "]";
+      << "Invalid header field for stream_id=" << stream_id
+      << " in frame type=" << static_cast<uint32_t>(frame->hd.type)
+      << ": name=[" << as_string_view(namebuf.base, namebuf.len) << "], value=["
+      << as_string_view(valuebuf.base, valuebuf.len) << "]";
   }
 
   http2session->submit_rst_stream(stream_id, NGHTTP2_PROTOCOL_ERROR);
@@ -1055,7 +1037,7 @@ int on_begin_headers_callback(nghttp2_session *session,
       return 0;
     }
     auto sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
     if (!sd || !sd->dconn) {
       http2session->submit_rst_stream(frame->hd.stream_id,
                                       NGHTTP2_INTERNAL_ERROR);
@@ -1066,7 +1048,7 @@ int on_begin_headers_callback(nghttp2_session *session,
   case NGHTTP2_PUSH_PROMISE: {
     auto promised_stream_id = frame->push_promise.promised_stream_id;
     auto sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
     if (!sd || !sd->dconn) {
       http2session->submit_rst_stream(promised_stream_id, NGHTTP2_CANCEL);
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
@@ -1113,12 +1095,12 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
   assert(status);
   auto status_code = http2::parse_http_status_code(status->value);
 
-  resp.http_status = status_code;
+  resp.http_status = as_unsigned(status_code);
   resp.http_major = 2;
   resp.http_minor = 0;
 
   downstream->set_downstream_addr_group(
-      http2session->get_downstream_addr_group());
+    http2session->get_downstream_addr_group());
   downstream->set_addr(http2session->get_addr());
 
   if (LOG_ENABLED(INFO)) {
@@ -1127,12 +1109,11 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
       ss << TTY_HTTP_HD << nv.name << TTY_RST << ": " << nv.value << "\n";
     }
     SSLOG(INFO, http2session)
-        << "HTTP response headers. stream_id=" << frame->hd.stream_id << "\n"
-        << ss.str();
+      << "HTTP response headers. stream_id=" << frame->hd.stream_id << "\n"
+      << ss.str();
   }
 
   if (downstream->get_non_final_response()) {
-
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, http2session) << "This is non-final response.";
     }
@@ -1165,13 +1146,14 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
     downstream->set_request_state(DownstreamState::HEADER_COMPLETE);
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, http2session)
-          << "HTTP upgrade success. stream_id=" << frame->hd.stream_id;
+        << "HTTP upgrade success. stream_id=" << frame->hd.stream_id;
     }
   } else {
     auto content_length = resp.fs.header(http2::HD_CONTENT_LENGTH);
     if (content_length) {
       // libnghttp2 guarantees this can be parsed
-      resp.fs.content_length = util::parse_uint(content_length->value);
+      resp.fs.content_length =
+        util::parse_uint(content_length->value).value_or(-1);
     }
 
     if (resp.fs.content_length == -1 && downstream->expect_response_body()) {
@@ -1184,8 +1166,7 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
         // Otherwise, use chunked encoding to keep upstream connection
         // open.  In HTTP2, we are supposed not to receive
         // transfer-encoding.
-        resp.fs.add_header_token(StringRef::from_lit("transfer-encoding"),
-                                 StringRef::from_lit("chunked"), false,
+        resp.fs.add_header_token("transfer-encoding"sv, "chunked"sv, false,
                                  http2::HD_TRANSFER_ENCODING);
         downstream->set_chunked_response(true);
       }
@@ -1227,7 +1208,7 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
   switch (frame->hd.type) {
   case NGHTTP2_DATA: {
     auto sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
     if (!sd || !sd->dconn) {
       return 0;
     }
@@ -1240,12 +1221,10 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
       downstream->set_response_state(DownstreamState::MSG_RESET);
 
     } else if (frame->hd.flags & NGHTTP2_FLAG_END_STREAM) {
-
       downstream->disable_downstream_rtimer();
 
       if (downstream->get_response_state() ==
           DownstreamState::HEADER_COMPLETE) {
-
         downstream->set_response_state(DownstreamState::MSG_COMPLETE);
 
         rv = upstream->on_downstream_body_complete(downstream);
@@ -1261,7 +1240,7 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
   }
   case NGHTTP2_HEADERS: {
     auto sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
     if (!sd || !sd->dconn) {
       return 0;
     }
@@ -1310,11 +1289,11 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
   }
   case NGHTTP2_RST_STREAM: {
     auto sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
     if (sd && sd->dconn) {
       auto downstream = sd->dconn->get_downstream();
       downstream->set_response_rst_stream_error_code(
-          frame->rst_stream.error_code);
+        frame->rst_stream.error_code);
       call_downstream_readcb(http2session, downstream);
     }
     return 0;
@@ -1347,13 +1326,12 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
 
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, http2session)
-          << "Received downstream PUSH_PROMISE stream_id="
-          << frame->hd.stream_id
-          << ", promised_stream_id=" << promised_stream_id;
+        << "Received downstream PUSH_PROMISE stream_id=" << frame->hd.stream_id
+        << ", promised_stream_id=" << promised_stream_id;
     }
 
     auto sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
     if (!sd || !sd->dconn) {
       http2session->submit_rst_stream(promised_stream_id, NGHTTP2_CANCEL);
       return 0;
@@ -1365,7 +1343,7 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
     assert(downstream->get_downstream_stream_id() == frame->hd.stream_id);
 
     auto promised_sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, promised_stream_id));
+      nghttp2_session_get_stream_user_data(session, promised_stream_id));
     if (!promised_sd || !promised_sd->dconn) {
       http2session->submit_rst_stream(promised_stream_id, NGHTTP2_CANCEL);
       return 0;
@@ -1376,7 +1354,7 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
     assert(promised_downstream);
 
     if (http2session->handle_downstream_push_promise_complete(
-            downstream, promised_downstream) != 0) {
+          downstream, promised_downstream) != 0) {
       http2session->submit_rst_stream(promised_stream_id, NGHTTP2_CANCEL);
       return 0;
     }
@@ -1389,9 +1367,9 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
                                          frame->goaway.opaque_data_len);
 
       SSLOG(INFO, http2session)
-          << "GOAWAY received: last-stream-id=" << frame->goaway.last_stream_id
-          << ", error_code=" << frame->goaway.error_code
-          << ", debug_data=" << debug_data;
+        << "GOAWAY received: last-stream-id=" << frame->goaway.last_stream_id
+        << ", error_code=" << frame->goaway.error_code
+        << ", debug_data=" << debug_data;
     }
     return 0;
   default:
@@ -1407,7 +1385,7 @@ int on_data_chunk_recv_callback(nghttp2_session *session, uint8_t flags,
   int rv;
   auto http2session = static_cast<Http2Session *>(user_data);
   auto sd = static_cast<StreamData *>(
-      nghttp2_session_get_stream_user_data(session, stream_id));
+    nghttp2_session_get_stream_user_data(session, stream_id));
   if (!sd || !sd->dconn) {
     http2session->submit_rst_stream(stream_id, NGHTTP2_INTERNAL_ERROR);
 
@@ -1471,7 +1449,7 @@ int on_frame_send_callback(nghttp2_session *session, const nghttp2_frame *frame,
 
   if (frame->hd.type == NGHTTP2_DATA || frame->hd.type == NGHTTP2_HEADERS) {
     auto sd = static_cast<StreamData *>(
-        nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
 
     if (!sd || !sd->dconn) {
       return 0;
@@ -1528,7 +1506,7 @@ int on_frame_not_send_callback(nghttp2_session *session,
   }
 
   auto sd = static_cast<StreamData *>(
-      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+    nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
   if (!sd) {
     return 0;
   }
@@ -1568,7 +1546,7 @@ int send_data_callback(nghttp2_session *session, nghttp2_frame *frame,
                        nghttp2_data_source *source, void *user_data) {
   auto http2session = static_cast<Http2Session *>(user_data);
   auto sd = static_cast<StreamData *>(
-      nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
+    nghttp2_session_get_stream_user_data(session, frame->hd.stream_id));
 
   if (sd == nullptr) {
     return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
@@ -1584,7 +1562,7 @@ int send_data_callback(nghttp2_session *session, nghttp2_frame *frame,
   wb->append(framehd, 9);
   if (frame->data.padlen > 0) {
     padlen = frame->data.padlen - 1;
-    wb->append(static_cast<uint8_t>(padlen));
+    wb->append(static_cast<char>(padlen));
   }
 
   input->remove(*wb, length);
@@ -1626,35 +1604,35 @@ nghttp2_session_callbacks *create_http2_downstream_callbacks() {
   }
 
   nghttp2_session_callbacks_set_on_stream_close_callback(
-      callbacks, on_stream_close_callback);
+    callbacks, on_stream_close_callback);
 
   nghttp2_session_callbacks_set_on_frame_recv_callback(callbacks,
                                                        on_frame_recv_callback);
 
   nghttp2_session_callbacks_set_on_data_chunk_recv_callback(
-      callbacks, on_data_chunk_recv_callback);
+    callbacks, on_data_chunk_recv_callback);
 
   nghttp2_session_callbacks_set_on_frame_send_callback(callbacks,
                                                        on_frame_send_callback);
 
   nghttp2_session_callbacks_set_on_frame_not_send_callback(
-      callbacks, on_frame_not_send_callback);
+    callbacks, on_frame_not_send_callback);
 
   nghttp2_session_callbacks_set_on_header_callback2(callbacks,
                                                     on_header_callback2);
 
   nghttp2_session_callbacks_set_on_invalid_header_callback2(
-      callbacks, on_invalid_header_callback2);
+    callbacks, on_invalid_header_callback2);
 
   nghttp2_session_callbacks_set_on_begin_headers_callback(
-      callbacks, on_begin_headers_callback);
+    callbacks, on_begin_headers_callback);
 
   nghttp2_session_callbacks_set_send_data_callback(callbacks,
                                                    send_data_callback);
 
   if (get_config()->padding) {
     nghttp2_session_callbacks_set_select_padding_callback2(
-        callbacks, http::select_padding_callback);
+      callbacks, http::select_padding_callback);
   }
 
   return callbacks;
@@ -1679,7 +1657,7 @@ int Http2Session::connection_made() {
       return -1;
     }
 
-    auto proto = StringRef{next_proto, next_proto_len};
+    auto proto = as_string_view(next_proto, next_proto_len);
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, this) << "Negotiated next protocol: " << proto;
     }
@@ -1702,10 +1680,11 @@ int Http2Session::connection_made() {
   std::array<nghttp2_settings_entry, 5> entry;
   size_t nentry = 3;
   entry[0].settings_id = NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS;
-  entry[0].value = http2conf.downstream.max_concurrent_streams;
+  entry[0].value =
+    static_cast<uint32_t>(http2conf.downstream.max_concurrent_streams);
 
   entry[1].settings_id = NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE;
-  entry[1].value = http2conf.downstream.window_size;
+  entry[1].value = as_unsigned(http2conf.downstream.window_size);
 
   entry[2].settings_id = NGHTTP2_SETTINGS_NO_RFC7540_PRIORITIES;
   entry[2].value = 1;
@@ -1719,19 +1698,20 @@ int Http2Session::connection_made() {
   if (http2conf.downstream.decoder_dynamic_table_size !=
       NGHTTP2_DEFAULT_HEADER_TABLE_SIZE) {
     entry[nentry].settings_id = NGHTTP2_SETTINGS_HEADER_TABLE_SIZE;
-    entry[nentry].value = http2conf.downstream.decoder_dynamic_table_size;
+    entry[nentry].value =
+      static_cast<uint32_t>(http2conf.downstream.decoder_dynamic_table_size);
     ++nentry;
   }
 
-  rv = nghttp2_submit_settings(session_, NGHTTP2_FLAG_NONE, entry.data(),
-                               nentry);
+  rv =
+    nghttp2_submit_settings(session_, NGHTTP2_FLAG_NONE, entry.data(), nentry);
   if (rv != 0) {
     return -1;
   }
 
   rv = nghttp2_session_set_local_window_size(
-      session_, NGHTTP2_FLAG_NONE, 0,
-      http2conf.downstream.connection_window_size);
+    session_, NGHTTP2_FLAG_NONE, 0,
+    http2conf.downstream.connection_window_size);
   if (rv != 0) {
     return -1;
   }
@@ -1757,7 +1737,7 @@ int Http2Session::downstream_read(const uint8_t *data, size_t datalen) {
   auto rv = nghttp2_session_mem_recv2(session_, data, datalen);
   if (rv < 0) {
     SSLOG(ERROR, this) << "nghttp2_session_mem_recv2() returned error: "
-                       << nghttp2_strerror(rv);
+                       << nghttp2_strerror(static_cast<int>(rv));
     return -1;
   }
 
@@ -1779,13 +1759,13 @@ int Http2Session::downstream_write() {
     auto datalen = nghttp2_session_mem_send2(session_, &data);
     if (datalen < 0) {
       SSLOG(ERROR, this) << "nghttp2_session_mem_send2() returned error: "
-                         << nghttp2_strerror(datalen);
+                         << nghttp2_strerror(static_cast<int>(datalen));
       return -1;
     }
     if (datalen == 0) {
       break;
     }
-    wb_.append(data, datalen);
+    wb_.append(data, as_unsigned(datalen));
 
     if (wb_.rleft() >= MAX_BUFFER_SIZE) {
       break;
@@ -2019,10 +1999,10 @@ int Http2Session::read_clear() {
     }
 
     if (nread < 0) {
-      return nread;
+      return static_cast<int>(nread);
     }
 
-    if (on_read(buf.data(), nread) != 0) {
+    if (on_read(buf.data(), as_unsigned(nread)) != 0) {
       return -1;
     }
   }
@@ -2051,7 +2031,7 @@ int Http2Session::write_clear() {
         break;
       }
 
-      wb_.drain(nwrite);
+      wb_.drain(as_unsigned(nwrite));
       continue;
     }
 
@@ -2123,10 +2103,10 @@ int Http2Session::read_tls() {
     }
 
     if (nread < 0) {
-      return nread;
+      return static_cast<int>(nread);
     }
 
-    if (on_read(buf.data(), nread) != 0) {
+    if (on_read(buf.data(), as_unsigned(nread)) != 0) {
       return -1;
     }
   }
@@ -2161,7 +2141,7 @@ int Http2Session::write_tls() {
         break;
       }
 
-      wb_.drain(nwrite);
+      wb_.drain(as_unsigned(nwrite));
 
       continue;
     }
@@ -2210,7 +2190,7 @@ int Http2Session::handle_downstream_push_promise(Downstream *downstream,
   }
 
   auto promised_downstream =
-      upstream->on_downstream_push_promise(downstream, promised_stream_id);
+    upstream->on_downstream_push_promise(downstream, promised_stream_id);
   if (!promised_downstream) {
     return -1;
   }
@@ -2226,7 +2206,7 @@ int Http2Session::handle_downstream_push_promise(Downstream *downstream,
   auto ptr = promised_dconn.get();
 
   if (promised_downstream->attach_downstream_connection(
-          std::move(promised_dconn)) != 0) {
+        std::move(promised_dconn)) != 0) {
     return -1;
   }
 
@@ -2242,7 +2222,7 @@ int Http2Session::handle_downstream_push_promise(Downstream *downstream,
 }
 
 int Http2Session::handle_downstream_push_promise_complete(
-    Downstream *downstream, Downstream *promised_downstream) {
+  Downstream *downstream, Downstream *promised_downstream) {
   auto &promised_req = promised_downstream->request();
 
   auto &promised_balloc = promised_downstream->get_block_allocator();
@@ -2279,7 +2259,7 @@ int Http2Session::handle_downstream_push_promise_complete(
   }
 
   // For server-wide OPTIONS request, path is empty.
-  if (method_token != HTTP_OPTIONS || path->value != "*") {
+  if (method_token != HTTP_OPTIONS || path->value != "*"sv) {
     promised_req.path = http2::rewrite_clean_path(promised_balloc, path->value);
   }
 
@@ -2309,8 +2289,8 @@ bool Http2Session::max_concurrency_reached(size_t extra) const {
   // that maximum concurrency is reached.
   return !nghttp2_session_check_request_allowed(session_) ||
          dconns_.size() + extra >=
-             nghttp2_session_get_remote_settings(
-                 session_, NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS);
+           nghttp2_session_get_remote_settings(
+             session_, NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS);
 }
 
 const std::shared_ptr<DownstreamAddrGroup> &
