@@ -25,7 +25,6 @@
 #include "shrpx_https_upstream.h"
 
 #include <cassert>
-#include <set>
 #include <sstream>
 
 #include "shrpx_client_handler.h"
@@ -45,7 +44,7 @@
 #include "util.h"
 #include "template.h"
 #include "base64.h"
-#include "url-parser/url_parser.h"
+#include "urlparse.h"
 
 using namespace nghttp2;
 
@@ -63,37 +62,21 @@ int htp_msg_completecb(llhttp_t *htp);
 
 namespace {
 constexpr llhttp_settings_t htp_hooks = {
-    htp_msg_begin,       // llhttp_cb      on_message_begin;
-    htp_uricb,           // llhttp_data_cb on_url;
-    nullptr,             // llhttp_data_cb on_status;
-    nullptr,             // llhttp_data_cb on_method;
-    nullptr,             // llhttp_data_cb on_version;
-    htp_hdr_keycb,       // llhttp_data_cb on_header_field;
-    htp_hdr_valcb,       // llhttp_data_cb on_header_value;
-    nullptr,             // llhttp_data_cb on_chunk_extension_name;
-    nullptr,             // llhttp_data_cb on_chunk_extension_value;
-    htp_hdrs_completecb, // llhttp_cb      on_headers_complete;
-    htp_bodycb,          // llhttp_data_cb on_body;
-    htp_msg_completecb,  // llhttp_cb      on_message_complete;
-    nullptr,             // llhttp_cb      on_url_complete;
-    nullptr,             // llhttp_cb      on_status_complete;
-    nullptr,             // llhttp_cb      on_method_complete;
-    nullptr,             // llhttp_cb      on_version_complete;
-    nullptr,             // llhttp_cb      on_header_field_complete;
-    nullptr,             // llhttp_cb      on_header_value_complete;
-    nullptr,             // llhttp_cb      on_chunk_extension_name_complete;
-    nullptr,             // llhttp_cb      on_chunk_extension_value_complete;
-    nullptr,             // llhttp_cb      on_chunk_header;
-    nullptr,             // llhttp_cb      on_chunk_complete;
-    nullptr,             // llhttp_cb      on_reset;
+  .on_message_begin = htp_msg_begin,
+  .on_url = htp_uricb,
+  .on_header_field = htp_hdr_keycb,
+  .on_header_value = htp_hdr_valcb,
+  .on_headers_complete = htp_hdrs_completecb,
+  .on_body = htp_bodycb,
+  .on_message_complete = htp_msg_completecb,
 };
 } // namespace
 
 HttpsUpstream::HttpsUpstream(ClientHandler *handler)
-    : handler_(handler),
-      current_header_length_(0),
-      ioctrl_(handler->get_rlimit()),
-      num_requests_(0) {
+  : handler_(handler),
+    current_header_length_(0),
+    ioctrl_(handler->get_rlimit()),
+    num_requests_(0) {
   llhttp_init(&htp_, HTTP_REQUEST, &htp_hooks);
   htp_.data = this;
 }
@@ -111,7 +94,7 @@ void HttpsUpstream::on_start_request() {
   reset_current_header_length();
 
   auto downstream =
-      std::make_unique<Downstream>(this, handler_->get_mcpool(), 0);
+    std::make_unique<Downstream>(this, handler_->get_mcpool(), 0);
 
   attach_downstream(std::move(downstream));
 
@@ -149,7 +132,7 @@ int htp_uricb(llhttp_t *htp, const char *data, size_t len) {
     }
     assert(downstream->get_request_state() == DownstreamState::INITIAL);
     downstream->set_request_state(
-        DownstreamState::HTTP1_REQUEST_HEADER_TOO_LARGE);
+      DownstreamState::HTTP1_REQUEST_HEADER_TOO_LARGE);
     llhttp_set_error_reason(htp, "too long request URI");
     return HPE_USER;
   }
@@ -158,9 +141,9 @@ int htp_uricb(llhttp_t *htp, const char *data, size_t len) {
 
   if (req.method == HTTP_CONNECT) {
     req.authority =
-        concat_string_ref(balloc, req.authority, StringRef{data, len});
+      concat_string_ref(balloc, req.authority, std::string_view{data, len});
   } else {
-    req.path = concat_string_ref(balloc, req.path, StringRef{data, len});
+    req.path = concat_string_ref(balloc, req.path, std::string_view{data, len});
   }
 
   return 0;
@@ -181,41 +164,44 @@ int htp_hdr_keycb(llhttp_t *htp, const char *data, size_t len) {
     }
     if (downstream->get_request_state() == DownstreamState::INITIAL) {
       downstream->set_request_state(
-          DownstreamState::HTTP1_REQUEST_HEADER_TOO_LARGE);
+        DownstreamState::HTTP1_REQUEST_HEADER_TOO_LARGE);
     }
     llhttp_set_error_reason(htp, "too large header");
     return HPE_USER;
   }
+
+  auto name = std::string_view{data, len};
+
   if (downstream->get_request_state() == DownstreamState::INITIAL) {
     if (req.fs.header_key_prev()) {
-      req.fs.append_last_header_key(data, len);
+      req.fs.append_last_header_key(name);
     } else {
       if (req.fs.num_fields() >= httpconf.max_request_header_fields) {
         if (LOG_ENABLED(INFO)) {
           ULOG(INFO, upstream)
-              << "Too many header field num=" << req.fs.num_fields() + 1;
+            << "Too many header field num=" << req.fs.num_fields() + 1;
         }
         downstream->set_request_state(
-            DownstreamState::HTTP1_REQUEST_HEADER_TOO_LARGE);
+          DownstreamState::HTTP1_REQUEST_HEADER_TOO_LARGE);
         llhttp_set_error_reason(htp, "too many headers");
         return HPE_USER;
       }
-      req.fs.alloc_add_header_name(StringRef{data, len});
+      req.fs.alloc_add_header_name(name);
     }
   } else {
     // trailer part
     if (req.fs.trailer_key_prev()) {
-      req.fs.append_last_trailer_key(data, len);
+      req.fs.append_last_trailer_key(name);
     } else {
       if (req.fs.num_fields() >= httpconf.max_request_header_fields) {
         if (LOG_ENABLED(INFO)) {
           ULOG(INFO, upstream)
-              << "Too many header field num=" << req.fs.num_fields() + 1;
+            << "Too many header field num=" << req.fs.num_fields() + 1;
         }
         llhttp_set_error_reason(htp, "too many headers");
         return HPE_USER;
       }
-      req.fs.alloc_add_trailer_name(StringRef{data, len});
+      req.fs.alloc_add_trailer_name(name);
     }
   }
   return 0;
@@ -236,15 +222,18 @@ int htp_hdr_valcb(llhttp_t *htp, const char *data, size_t len) {
     }
     if (downstream->get_request_state() == DownstreamState::INITIAL) {
       downstream->set_request_state(
-          DownstreamState::HTTP1_REQUEST_HEADER_TOO_LARGE);
+        DownstreamState::HTTP1_REQUEST_HEADER_TOO_LARGE);
     }
     llhttp_set_error_reason(htp, "too large header");
     return HPE_USER;
   }
+
+  auto value = std::string_view{data, len};
+
   if (downstream->get_request_state() == DownstreamState::INITIAL) {
-    req.fs.append_last_header_value(data, len);
+    req.fs.append_last_header_value(value);
   } else {
-    req.fs.append_last_trailer_value(data, len);
+    req.fs.append_last_trailer_value(value);
   }
   return 0;
 }
@@ -252,50 +241,49 @@ int htp_hdr_valcb(llhttp_t *htp, const char *data, size_t len) {
 
 namespace {
 void rewrite_request_host_path_from_uri(BlockAllocator &balloc, Request &req,
-                                        const StringRef &uri,
-                                        http_parser_url &u) {
-  assert(u.field_set & (1 << UF_HOST));
+                                        const std::string_view &uri,
+                                        urlparse_url &u) {
+  assert(u.field_set & (1 << URLPARSE_HOST));
 
   // As per https://tools.ietf.org/html/rfc7230#section-5.4, we
   // rewrite host header field with authority component.
-  auto authority = util::get_uri_field(uri.c_str(), u, UF_HOST);
+  auto authority = util::get_uri_field(uri.data(), u, URLPARSE_HOST);
   // TODO properly check IPv6 numeric address
-  auto ipv6 = std::find(std::begin(authority), std::end(authority), ':') !=
-              std::end(authority);
+  auto ipv6 = util::contains(authority, ':');
   auto authoritylen = authority.size();
   if (ipv6) {
     authoritylen += 2;
   }
-  if (u.field_set & (1 << UF_PORT)) {
+  if (u.field_set & (1 << URLPARSE_PORT)) {
     authoritylen += 1 + str_size("65535");
   }
   if (authoritylen > authority.size()) {
     auto iovec = make_byte_ref(balloc, authoritylen + 1);
-    auto p = iovec.base;
+    auto p = std::ranges::begin(iovec);
     if (ipv6) {
       *p++ = '[';
     }
-    p = std::copy(std::begin(authority), std::end(authority), p);
+    p = std::ranges::copy(authority, p).out;
     if (ipv6) {
       *p++ = ']';
     }
 
-    if (u.field_set & (1 << UF_PORT)) {
+    if (u.field_set & (1 << URLPARSE_PORT)) {
       *p++ = ':';
-      p = util::utos(p, u.port);
+      p = util::utos(u.port, p);
     }
     *p = '\0';
 
-    req.authority = StringRef{iovec.base, p};
+    req.authority = as_string_view(std::ranges::begin(iovec), p);
   } else {
     req.authority = authority;
   }
 
-  req.scheme = util::get_uri_field(uri.c_str(), u, UF_SCHEMA);
+  req.scheme = util::get_uri_field(uri.data(), u, URLPARSE_SCHEMA);
 
-  StringRef path;
-  if (u.field_set & (1 << UF_PATH)) {
-    path = util::get_uri_field(uri.c_str(), u, UF_PATH);
+  std::string_view path;
+  if (u.field_set & (1 << URLPARSE_PATH)) {
+    path = util::get_uri_field(uri.data(), u, URLPARSE_PATH);
   } else if (req.method == HTTP_OPTIONS) {
     // Server-wide OPTIONS takes following form in proxy request:
     //
@@ -303,22 +291,22 @@ void rewrite_request_host_path_from_uri(BlockAllocator &balloc, Request &req,
     //
     // Notice that no slash after authority. See
     // http://tools.ietf.org/html/rfc7230#section-5.3.4
-    req.path = StringRef::from_lit("");
+    req.path = ""sv;
     // we ignore query component here
     return;
   } else {
-    path = StringRef::from_lit("/");
+    path = "/"sv;
   }
 
-  if (u.field_set & (1 << UF_QUERY)) {
-    auto &fdata = u.field_data[UF_QUERY];
+  if (u.field_set & (1 << URLPARSE_QUERY)) {
+    auto &fdata = u.field_data[URLPARSE_QUERY];
 
-    if (u.field_set & (1 << UF_PATH)) {
-      auto q = util::get_uri_field(uri.c_str(), u, UF_QUERY);
-      path = StringRef{std::begin(path), std::end(q)};
+    if (u.field_set & (1 << URLPARSE_PATH)) {
+      auto q = util::get_uri_field(uri.data(), u, URLPARSE_QUERY);
+      path = std::string_view{std::ranges::begin(path), std::ranges::end(q)};
     } else {
-      path = concat_string_ref(balloc, path, StringRef::from_lit("?"),
-                               StringRef{&uri[fdata.off], fdata.len});
+      path = concat_string_ref(balloc, path, "?"sv,
+                               std::string_view{&uri[fdata.off], fdata.len});
     }
   }
 
@@ -369,7 +357,7 @@ int htp_hdrs_completecb(llhttp_t *htp) {
        << "HTTP/" << req.http_major << "." << req.http_minor << "\n";
 
     for (const auto &kv : req.fs.headers()) {
-      if (kv.name == "authorization") {
+      if (kv.name == "authorization"sv) {
         ss << TTY_HTTP_HD << kv.name << TTY_RST << ": <redacted>\n";
         continue;
       }
@@ -387,7 +375,11 @@ int htp_hdrs_completecb(llhttp_t *htp) {
     // content-length header field.  If we don't have both
     // transfer-encoding and content-length header field, we assume
     // that there is no request body.
-    req.fs.content_length = htp->content_length;
+    if (htp->content_length > std::numeric_limits<int64_t>::max()) {
+      return -1;
+    }
+
+    req.fs.content_length = static_cast<int64_t>(htp->content_length);
   }
 
   auto host = req.fs.header(http2::HD_HOST);
@@ -407,9 +399,9 @@ int htp_hdrs_completecb(llhttp_t *htp) {
     // Not allow at least '"' or '\' in host.  They are illegal in
     // authority component, also they cause headaches when we put them
     // in quoted-string.
-    if (std::find_if(std::begin(value), std::end(value), [](char c) {
+    if (std::ranges::find_if(value, [](char c) {
           return c == '"' || c == '\\';
-        }) != std::end(value)) {
+        }) != std::ranges::end(value)) {
       return -1;
     }
   }
@@ -430,18 +422,19 @@ int htp_hdrs_completecb(llhttp_t *htp) {
   auto config = get_config();
 
   if (method != HTTP_CONNECT) {
-    http_parser_url u{};
-    rv = http_parser_parse_url(req.path.c_str(), req.path.size(), 0, &u);
+    urlparse_url u;
+    rv = urlparse_parse_url(req.path.data(), req.path.size(), 0, &u);
     if (rv != 0) {
       // Expect to respond with 400 bad request
       return -1;
     }
-    // checking UF_HOST could be redundant, but just in case ...
-    if (!(u.field_set & (1 << UF_SCHEMA)) || !(u.field_set & (1 << UF_HOST))) {
+    // checking URLPARSE_HOST could be redundant, but just in case ...
+    if (!(u.field_set & (1 << URLPARSE_SCHEMA)) ||
+        !(u.field_set & (1 << URLPARSE_HOST))) {
       req.no_authority = true;
 
-      if (method == HTTP_OPTIONS && req.path == StringRef::from_lit("*")) {
-        req.path = StringRef{};
+      if (method == HTTP_OPTIONS && req.path == "*"sv) {
+        req.path = ""sv;
       } else {
         req.path = http2::rewrite_clean_path(balloc, req.path);
       }
@@ -451,9 +444,9 @@ int htp_hdrs_completecb(llhttp_t *htp) {
       }
 
       if (handler->get_ssl()) {
-        req.scheme = StringRef::from_lit("https");
+        req.scheme = "https"sv;
       } else {
-        req.scheme = StringRef::from_lit("http");
+        req.scheme = "http"sv;
       }
     } else {
       rewrite_request_host_path_from_uri(balloc, req, req.path, u);
@@ -544,7 +537,7 @@ int htp_hdrs_completecb(llhttp_t *htp) {
     // Continue here to make the client happy.
     if (downstream->get_expect_100_continue()) {
       auto output = downstream->get_response_buf();
-      constexpr auto res = StringRef::from_lit("HTTP/1.1 100 Continue\r\n\r\n");
+      constexpr auto res = "HTTP/1.1 100 Continue\r\n\r\n"sv;
       output->append(res);
       handler->signal_write();
     }
@@ -560,7 +553,7 @@ int htp_bodycb(llhttp_t *htp, const char *data, size_t len) {
   auto upstream = static_cast<HttpsUpstream *>(htp->data);
   auto downstream = upstream->get_downstream();
   rv = downstream->push_upload_data_chunk(
-      reinterpret_cast<const uint8_t *>(data), len);
+    reinterpret_cast<const uint8_t *>(data), len);
   if (rv != 0) {
     // Ignore error if response has been completed.  We will end up in
     // htp_msg_completecb, and request will end gracefully.
@@ -633,7 +626,6 @@ int HttpsUpstream::on_read() {
   // downstream can be nullptr here, because it is initialized in the
   // callback chain called by llhttp_execute()
   if (downstream && downstream->get_upgraded()) {
-
     auto rv = downstream->push_upload_data_chunk(rb->pos(), rb->rleft());
 
     if (rv != 0) {
@@ -672,18 +664,17 @@ int HttpsUpstream::on_read() {
 
   if (htperr == HPE_PAUSED_UPGRADE &&
       rb->pos() ==
-          reinterpret_cast<const uint8_t *>(llhttp_get_error_pos(&htp_))) {
+        reinterpret_cast<const uint8_t *>(llhttp_get_error_pos(&htp_))) {
     llhttp_resume_after_upgrade(&htp_);
 
     htperr = llhttp_execute(&htp_, reinterpret_cast<const char *>(rb->pos()),
                             rb->rleft());
   }
 
-  auto nread =
-      htperr == HPE_OK
-          ? rb->rleft()
-          : reinterpret_cast<const uint8_t *>(llhttp_get_error_pos(&htp_)) -
-                rb->pos();
+  auto nread = htperr == HPE_OK ? rb->rleft()
+                                : as_unsigned(reinterpret_cast<const uint8_t *>(
+                                                llhttp_get_error_pos(&htp_)) -
+                                              rb->pos());
   rb->drain(nread);
   rlimit->startw();
 
@@ -982,8 +973,7 @@ int HttpsUpstream::send_reply(Downstream *downstream, const uint8_t *body,
 
   if (httpconf.max_requests <= num_requests_ ||
       worker->get_graceful_shutdown()) {
-    resp.fs.add_header_token(StringRef::from_lit("connection"),
-                             StringRef::from_lit("close"), false,
+    resp.fs.add_header_token("connection"sv, "close"sv, false,
                              http2::HD_CONNECTION);
     connection_close = true;
   } else if (req.http_major <= 0 ||
@@ -991,7 +981,7 @@ int HttpsUpstream::send_reply(Downstream *downstream, const uint8_t *body,
     connection_close = true;
   } else {
     auto c = resp.fs.header(http2::HD_CONNECTION);
-    if (c && util::strieq_l("close", c->value)) {
+    if (c && util::strieq("close"sv, c->value)) {
       connection_close = true;
     }
   }
@@ -1003,40 +993,43 @@ int HttpsUpstream::send_reply(Downstream *downstream, const uint8_t *body,
 
   auto output = downstream->get_response_buf();
 
-  output->append("HTTP/1.1 ");
+  output->append("HTTP/1.1 "sv);
   output->append(http2::stringify_status(balloc, resp.http_status));
   output->append(' ');
   output->append(http2::get_reason_phrase(resp.http_status));
-  output->append("\r\n");
+  output->append("\r\n"sv);
 
   for (auto &kv : resp.fs.headers()) {
     if (kv.name.empty() || kv.name[0] == ':') {
       continue;
     }
     http2::capitalize(output, kv.name);
-    output->append(": ");
+    output->append(": "sv);
     output->append(kv.value);
-    output->append("\r\n");
+    output->append("\r\n"sv);
   }
 
   if (!resp.fs.header(http2::HD_SERVER)) {
-    output->append("Server: ");
+    output->append("Server: "sv);
     output->append(config->http.server_name);
-    output->append("\r\n");
+    output->append("\r\n"sv);
   }
 
   for (auto &p : httpconf.add_response_headers) {
     output->append(p.name);
-    output->append(": ");
+    output->append(": "sv);
     output->append(p.value);
-    output->append("\r\n");
+    output->append("\r\n"sv);
   }
 
-  output->append("\r\n");
+  output->append("\r\n"sv);
 
-  output->append(body, bodylen);
+  if (req.method != HTTP_HEAD) {
+    output->append(body, bodylen);
+
+    downstream->response_sent_body_length += bodylen;
+  }
 
-  downstream->response_sent_body_length += bodylen;
   downstream->set_response_state(DownstreamState::MSG_COMPLETE);
 
   return 0;
@@ -1047,7 +1040,7 @@ void HttpsUpstream::error_reply(unsigned int status_code) {
 
   if (!downstream) {
     attach_downstream(
-        std::make_unique<Downstream>(this, handler_->get_mcpool(), 1));
+      std::make_unique<Downstream>(this, handler_->get_mcpool(), 1));
     downstream = get_downstream();
   }
 
@@ -1064,25 +1057,30 @@ void HttpsUpstream::error_reply(unsigned int status_code) {
 
   auto output = downstream->get_response_buf();
 
-  output->append("HTTP/1.1 ");
+  output->append("HTTP/1.1 "sv);
   output->append(http2::stringify_status(balloc, status_code));
   output->append(' ');
   output->append(http2::get_reason_phrase(status_code));
-  output->append("\r\nServer: ");
+  output->append("\r\nServer: "sv);
   output->append(get_config()->http.server_name);
-  output->append("\r\nContent-Length: ");
-  std::array<uint8_t, NGHTTP2_MAX_UINT64_DIGITS> intbuf;
-  output->append(StringRef{std::begin(intbuf),
-                           util::utos(std::begin(intbuf), html.size())});
-  output->append("\r\nDate: ");
+  output->append("\r\nContent-Length: "sv);
+  output->append(std::numeric_limits<decltype(html.size())>::digits10 + 1,
+                 std::bind_front(util::UIntFormatter{}, html.size()));
+  output->append("\r\nDate: "sv);
   auto lgconf = log_config();
   lgconf->update_tstamp(std::chrono::system_clock::now());
   output->append(lgconf->tstamp->time_http);
   output->append("\r\nContent-Type: text/html; "
-                 "charset=UTF-8\r\nConnection: close\r\n\r\n");
-  output->append(html);
+                 "charset=UTF-8\r\nConnection: close\r\n\r\n"sv);
+
+  const auto &req = downstream->request();
+
+  if (req.method != HTTP_HEAD) {
+    output->append(html);
+
+    downstream->response_sent_body_length += html.size();
+  }
 
-  downstream->response_sent_body_length += html.size();
   downstream->set_response_state(DownstreamState::MSG_COMPLETE);
 }
 
@@ -1160,35 +1158,33 @@ int HttpsUpstream::on_downstream_header_complete(Downstream *downstream) {
   auto connect_method = req.method == HTTP_CONNECT;
 
   auto buf = downstream->get_response_buf();
-  buf->append("HTTP/");
-  buf->append('0' + req.http_major);
+  buf->append("HTTP/"sv);
+  buf->append('0' + static_cast<char>(req.http_major));
   buf->append('.');
-  buf->append('0' + req.http_minor);
+  buf->append('0' + static_cast<char>(req.http_minor));
   buf->append(' ');
   if (req.connect_proto != ConnectProto::NONE && downstream->get_upgraded()) {
-    buf->append(http2::stringify_status(balloc, 101));
-    buf->append(' ');
-    buf->append(http2::get_reason_phrase(101));
+    buf->append("101 Switching Protocols"sv);
   } else {
     buf->append(http2::stringify_status(balloc, resp.http_status));
     buf->append(' ');
     buf->append(http2::get_reason_phrase(resp.http_status));
   }
-  buf->append("\r\n");
+  buf->append("\r\n"sv);
 
   auto config = get_config();
   auto &httpconf = config->http;
 
   if (!config->http2_proxy && !httpconf.no_location_rewrite) {
     downstream->rewrite_location_response_header(
-        get_client_handler()->get_upstream_scheme());
+      get_client_handler()->get_upstream_scheme());
   }
 
   if (downstream->get_non_final_response()) {
     http2::build_http1_headers_from_headers(buf, resp.fs.headers(),
                                             http2::HDOP_STRIP_ALL);
 
-    buf->append("\r\n");
+    buf->append("\r\n"sv);
 
     if (LOG_ENABLED(INFO)) {
       log_response_headers(buf);
@@ -1199,19 +1195,21 @@ int HttpsUpstream::on_downstream_header_complete(Downstream *downstream) {
     return 0;
   }
 
-  auto build_flags = (http2::HDOP_STRIP_ALL & ~http2::HDOP_STRIP_VIA) |
-                     (!http2::legacy_http1(req.http_major, req.http_minor)
-                          ? 0
-                          : http2::HDOP_STRIP_TRANSFER_ENCODING);
+  auto build_flags =
+    static_cast<uint32_t>((http2::HDOP_STRIP_ALL & ~http2::HDOP_STRIP_VIA) |
+                          (!http2::legacy_http1(req.http_major, req.http_minor)
+                             ? 0
+                             : http2::HDOP_STRIP_TRANSFER_ENCODING));
 
   http2::build_http1_headers_from_headers(buf, resp.fs.headers(), build_flags);
 
   auto worker = handler_->get_worker();
 
   // after graceful shutdown commenced, add connection: close header
-  // field.
+  // field.  If CONNECT request failed, close the connection.
   if (httpconf.max_requests <= num_requests_ ||
-      worker->get_graceful_shutdown()) {
+      worker->get_graceful_shutdown() ||
+      (connect_method && !downstream->get_upgraded())) {
     resp.connection_close = true;
   }
 
@@ -1220,16 +1218,16 @@ int HttpsUpstream::on_downstream_header_complete(Downstream *downstream) {
   if (!req.connection_close && !resp.connection_close) {
     if (req.http_major <= 0 || req.http_minor <= 0) {
       // We add this header for HTTP/1.0 or HTTP/0.9 clients
-      buf->append("Connection: Keep-Alive\r\n");
+      buf->append("Connection: Keep-Alive\r\n"sv);
     }
   } else if (!downstream->get_upgraded()) {
-    buf->append("Connection: close\r\n");
+    buf->append("Connection: close\r\n"sv);
   }
 
   if (!connect_method && downstream->get_upgraded()) {
     if (req.connect_proto == ConnectProto::WEBSOCKET &&
         resp.http_status / 100 == 2) {
-      buf->append("Upgrade: websocket\r\nConnection: Upgrade\r\n");
+      buf->append("Upgrade: websocket\r\nConnection: Upgrade\r\n"sv);
       auto key = req.fs.header(http2::HD_SEC_WEBSOCKET_KEY);
       if (!key || key->value.size() != base64::encode_length(16)) {
         return -1;
@@ -1239,22 +1237,22 @@ int HttpsUpstream::on_downstream_header_complete(Downstream *downstream) {
       if (accept.empty()) {
         return -1;
       }
-      buf->append("Sec-WebSocket-Accept: ");
+      buf->append("Sec-WebSocket-Accept: "sv);
       buf->append(accept);
-      buf->append("\r\n");
+      buf->append("\r\n"sv);
     } else {
       auto connection = resp.fs.header(http2::HD_CONNECTION);
       if (connection) {
-        buf->append("Connection: ");
+        buf->append("Connection: "sv);
         buf->append((*connection).value);
-        buf->append("\r\n");
+        buf->append("\r\n"sv);
       }
 
       auto upgrade = resp.fs.header(http2::HD_UPGRADE);
       if (upgrade) {
-        buf->append("Upgrade: ");
+        buf->append("Upgrade: "sv);
         buf->append((*upgrade).value);
-        buf->append("\r\n");
+        buf->append("\r\n"sv);
       }
     }
   }
@@ -1262,22 +1260,22 @@ int HttpsUpstream::on_downstream_header_complete(Downstream *downstream) {
   if (!resp.fs.header(http2::HD_ALT_SVC)) {
     // We won't change or alter alt-svc from backend for now
     if (!httpconf.altsvcs.empty()) {
-      buf->append("Alt-Svc: ");
+      buf->append("Alt-Svc: "sv);
       buf->append(httpconf.altsvc_header_value);
-      buf->append("\r\n");
+      buf->append("\r\n"sv);
     }
   }
 
   if (!config->http2_proxy && !httpconf.no_server_rewrite) {
-    buf->append("Server: ");
+    buf->append("Server: "sv);
     buf->append(httpconf.server_name);
-    buf->append("\r\n");
+    buf->append("\r\n"sv);
   } else {
     auto server = resp.fs.header(http2::HD_SERVER);
     if (server) {
-      buf->append("Server: ");
+      buf->append("Server: "sv);
       buf->append((*server).value);
-      buf->append("\r\n");
+      buf->append("\r\n"sv);
     }
   }
 
@@ -1288,43 +1286,42 @@ int HttpsUpstream::on_downstream_header_complete(Downstream *downstream) {
       auto &shared_addr = group->shared_addr;
       auto &cookieconf = shared_addr->affinity.cookie;
       auto secure =
-          http::require_cookie_secure_attribute(cookieconf.secure, req.scheme);
+        http::require_cookie_secure_attribute(cookieconf.secure, req.scheme);
       auto cookie_str = http::create_affinity_cookie(
-          balloc, cookieconf.name, affinity_cookie, cookieconf.path, secure);
-      buf->append("Set-Cookie: ");
+        balloc, cookieconf.name, affinity_cookie, cookieconf.path, secure);
+      buf->append("Set-Cookie: "sv);
       buf->append(cookie_str);
-      buf->append("\r\n");
+      buf->append("\r\n"sv);
     }
   }
 
   auto via = resp.fs.header(http2::HD_VIA);
   if (httpconf.no_via) {
     if (via) {
-      buf->append("Via: ");
+      buf->append("Via: "sv);
       buf->append((*via).value);
-      buf->append("\r\n");
+      buf->append("\r\n"sv);
     }
   } else {
-    buf->append("Via: ");
+    buf->append("Via: "sv);
     if (via) {
       buf->append((*via).value);
-      buf->append(", ");
+      buf->append(", "sv);
     }
-    std::array<char, 16> viabuf;
-    auto end = http::create_via_header_value(viabuf.data(), resp.http_major,
-                                             resp.http_minor);
-    buf->append(viabuf.data(), end - std::begin(viabuf));
-    buf->append("\r\n");
+
+    buf->append(16, std::bind_front(http::ViaValueGenerator{}, resp.http_major,
+                                    resp.http_minor));
+    buf->append("\r\n"sv);
   }
 
   for (auto &p : httpconf.add_response_headers) {
     buf->append(p.name);
-    buf->append(": ");
+    buf->append(": "sv);
     buf->append(p.value);
-    buf->append("\r\n");
+    buf->append("\r\n"sv);
   }
 
-  buf->append("\r\n");
+  buf->append("\r\n"sv);
 
   if (LOG_ENABLED(INFO)) {
     log_response_headers(buf);
@@ -1341,15 +1338,16 @@ int HttpsUpstream::on_downstream_body(Downstream *downstream,
   }
   auto output = downstream->get_response_buf();
   if (downstream->get_chunked_response()) {
-    output->append(util::utox(len));
-    output->append("\r\n");
+    output->append(sizeof(len) * 2,
+                   std::bind_front(util::CompactHexFormatter{}, len));
+    output->append("\r\n"sv);
   }
   output->append(data, len);
 
   downstream->response_sent_body_length += len;
 
   if (downstream->get_chunked_response()) {
-    output->append("\r\n");
+    output->append("\r\n"sv);
   }
   return 0;
 }
@@ -1362,12 +1360,12 @@ int HttpsUpstream::on_downstream_body_complete(Downstream *downstream) {
     auto output = downstream->get_response_buf();
     const auto &trailers = resp.fs.trailers();
     if (trailers.empty()) {
-      output->append("0\r\n\r\n");
+      output->append("0\r\n\r\n"sv);
     } else {
-      output->append("0\r\n");
+      output->append("0\r\n"sv);
       http2::build_http1_headers_from_headers(output, trailers,
                                               http2::HDOP_STRIP_ALL);
-      output->append("\r\n");
+      output->append("\r\n"sv);
     }
   }
   if (LOG_ENABLED(INFO)) {
@@ -1395,7 +1393,7 @@ int HttpsUpstream::on_downstream_abort_request(Downstream *downstream,
 }
 
 int HttpsUpstream::on_downstream_abort_request_with_https_redirect(
-    Downstream *downstream) {
+  Downstream *downstream) {
   redirect_to_https(downstream);
   handler_->signal_write();
   return 0;
@@ -1403,7 +1401,7 @@ int HttpsUpstream::on_downstream_abort_request_with_https_redirect(
 
 int HttpsUpstream::redirect_to_https(Downstream *downstream) {
   auto &req = downstream->request();
-  if (req.method == HTTP_CONNECT || req.scheme != "http" ||
+  if (req.method == HTTP_CONNECT || req.scheme != "http"sv ||
       req.authority.empty()) {
     error_reply(400);
     return 0;
@@ -1419,22 +1417,18 @@ int HttpsUpstream::redirect_to_https(Downstream *downstream) {
   auto config = get_config();
   auto &httpconf = config->http;
 
-  StringRef loc;
-  if (httpconf.redirect_https_port == StringRef::from_lit("443")) {
-    loc = concat_string_ref(balloc, StringRef::from_lit("https://"), authority,
-                            req.path);
+  std::string_view loc;
+  if (httpconf.redirect_https_port == "443"sv) {
+    loc = concat_string_ref(balloc, "https://"sv, authority, req.path);
   } else {
-    loc = concat_string_ref(balloc, StringRef::from_lit("https://"), authority,
-                            StringRef::from_lit(":"),
+    loc = concat_string_ref(balloc, "https://"sv, authority, ":"sv,
                             httpconf.redirect_https_port, req.path);
   }
 
   auto &resp = downstream->response();
   resp.http_status = 308;
-  resp.fs.add_header_token(StringRef::from_lit("location"), loc, false,
-                           http2::HD_LOCATION);
-  resp.fs.add_header_token(StringRef::from_lit("connection"),
-                           StringRef::from_lit("close"), false,
+  resp.fs.add_header_token("location"sv, loc, false, http2::HD_LOCATION);
+  resp.fs.add_header_token("connection"sv, "close"sv, false,
                            http2::HD_CONNECTION);
 
   return send_reply(downstream, nullptr, 0);
@@ -1446,7 +1440,7 @@ void HttpsUpstream::log_response_headers(DefaultMemchunks *buf) const {
     nhdrs.append(chunk->pos, chunk->last);
   }
   if (log_config()->errorlog_tty) {
-    nhdrs = http::colorizeHeaders(nhdrs.c_str());
+    nhdrs = http::colorize_headers(nhdrs);
   }
   ULOG(INFO, this) << "HTTP response headers\n" << nhdrs;
 }
@@ -1523,7 +1517,8 @@ fail:
   return 0;
 }
 
-int HttpsUpstream::initiate_push(Downstream *downstream, const StringRef &uri) {
+int HttpsUpstream::initiate_push(Downstream *downstream,
+                                 const std::string_view &uri) {
   return 0;
 }
 
@@ -1564,13 +1559,13 @@ HttpsUpstream::on_downstream_push_promise(Downstream *downstream,
 }
 
 int HttpsUpstream::on_downstream_push_promise_complete(
-    Downstream *downstream, Downstream *promised_downstream) {
+  Downstream *downstream, Downstream *promised_downstream) {
   return -1;
 }
 
 bool HttpsUpstream::push_enabled() const { return false; }
 
 void HttpsUpstream::cancel_premature_downstream(
-    Downstream *promised_downstream) {}
+  Downstream *promised_downstream) {}
 
 } // namespace shrpx
